package org.jvending.provisioning.dao.impl;

import org.hibernate.Session;
import org.hibernate.Query;
import org.hibernate.HibernateException;
import org.jvending.provisioning.model.clientbundle.*;
import org.jvending.provisioning.dao.ContentAccessorDAO;
import org.jvending.provisioning.dao.BaseProvisioningDAO;
import org.jvending.registry.RepositoryRegistry;

import java.io.IOException;
import java.util.List;
import java.util.Iterator;
import java.util.logging.Logger;
import java.sql.Blob;

public class ContentAccessorDAOImpl extends BaseProvisioningDAO implements ContentAccessorDAO {

    private static Logger logger = Logger.getLogger("DAO");

    private String dataSource = "hibernate3";

    public ContentAccessorDAOImpl(String className, String id) {
        super(className, id);
    }

    public void setRepositoryRegistry(RepositoryRegistry repositoryRegistry) {

    }

    public byte[] getDescriptorFor(String id) throws IOException {
        Session session = null;
        try {
            session = getSessionFactoryByName(dataSource).openSession();
            String queryStatement = "select clientbundle.descriptorFile.content from ClientBundle as clientbundle " +
                    "where bundle_id='" + id + "'";
            Query query = session.createQuery(queryStatement);
            List descriptors = query.list();
            Blob blob = (Blob) descriptors.get(0);
            return blob.getBytes(1L, (int) blob.length());
        } catch (Exception e) {
            e.printStackTrace();
            throw new IOException();
        } finally {
            try {
                if (session != null) session.close();
            } catch (HibernateException e) {
                e.printStackTrace();
                throw new IOException();
            }
        }
    }

    public byte[] getContentFor(String id) throws IOException {
        Session session = null;
        try {
            session = getSessionFactoryByName(dataSource).openSession();
            String queryStatement = "select clientbundle.contentFile.content from ClientBundle as clientbundle " +
                    "where bundle_id='" + id + "'";
            Query query = session.createQuery(queryStatement);
            List descriptors = query.list();
            Blob blob = (Blob) descriptors.get(0);
            return blob.getBytes(1L, (int) blob.length());
        } catch (Exception e) {
            e.printStackTrace();
            throw new IOException();
        } finally {
            try {
                if (session != null) session.close();
            } catch (HibernateException e) {
                e.printStackTrace();
                throw new IOException();
            }
        }
    }

    public byte[] getIconFor(String bundleId, String fileName) throws IOException {
        if (bundleId == null || fileName == null) throw new IOException("Invalid Parameters");

        Session session = null;
        try {
            session = getSessionFactoryByName(dataSource).openSession();
            String queryStatement = "select clientbundle from ClientBundle as clientbundle " +
                    "where bundle_id='" + bundleId + "'";
            Query query = session.createQuery(queryStatement);
            ClientBundle bundle = (ClientBundle) query.list().get(0);

            UserDescription description = bundle.getUserDescription();
            if (description == null) return null;

            List icons = description.getIcons();
            if (icons == null || icons.size() == 0) return null;

            for (Iterator i = icons.iterator(); i.hasNext();) {
                Icon icon = (Icon) i.next();
                String iconFileName = (String) icon.getFileUri();
                if (iconFileName == null) continue;
                if (iconFileName.equals(fileName)) {
                    Blob blob = icon.getFile();
                    return blob.getBytes(1L, (int) blob.length());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new IOException();
        } finally {
            try {
                if (session != null) session.close();
            } catch (HibernateException e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    public byte[] getPreviewFor(String id, String fileName) throws IOException {
        if (id == null || fileName == null) return null;

        Session session = null;
        try {
            session = getSessionFactoryByName(dataSource).openSession();
            String queryStatement = "select ClientBundle from ClientBundle as ClientBundle " +
                    "where bundle_id='" + id + "'";
            Query query = session.createQuery(queryStatement);
            ClientBundle bundle = (ClientBundle) query.list().get(0);

            List previews = bundle.getPreviews();
            if (previews == null || previews.size() == 0) return null;

            for (Iterator i = previews.iterator(); i.hasNext();) {
                Preview preview = (Preview) i.next();
                String fileUri = (String) preview.getFileUri();
                if (fileUri == null) continue;
                if (fileUri.equals(fileName)) {
                    Blob blob = preview.getFile();
                    return blob.getBytes(1L, (int) blob.length());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new IOException();
        } finally {
            try {
                if (session != null) session.close();
            } catch (HibernateException e) {
                e.printStackTrace();
                throw new IOException();
            }
        }
        return null;
    }

    public byte[] getDescriptorContentFor(String id, String fileName) throws IOException {
        if (id == null || fileName == null) return null;

        Session session = null;
        try {
            session = getSessionFactoryByName(dataSource).openSession();
            String queryStatement = "select ClientBundle from ClientBundle as ClientBundle " +
                    "where bundle_id='" + id + "'";
            Query query = session.createQuery(queryStatement);
            ClientBundle bundle = (ClientBundle) query.list().get(0);

            List contentFiles = bundle.getContentFilesFromDescriptor();
            if (contentFiles == null || contentFiles.size() == 0) {
                logger.finest("No content available from descriptor: Bundle Id = " + id +
                        ", File Name = " + fileName);
                return null;
            }

            for (Iterator i = contentFiles.iterator(); i.hasNext();) {
                ContentFile contentFile = (ContentFile) i.next();
                String fileUri = (String) contentFile.getFileUri();
                if (fileUri == null) continue;
                if (fileUri.equals(fileName)) {
                    Blob blob = contentFile.getContent();
                    return blob.getBytes(1L, (int) blob.length());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new IOException();
        } finally {
            try {
                if (session != null) session.close();
            } catch (HibernateException e) {
                e.printStackTrace();
                throw new IOException();
            }
        }
        return null;
    }
}
