package org.jvending.provisioning.stocking.filter;

import org.jvending.provisioning.stocking.par.ClientBundle;
import org.jvending.provisioning.stocking.par.DescriptorFile;
import org.jvending.provisioning.stocking.StockingException;
import org.jvending.provisioning.config.MimeTypeRepository;
import org.jvending.registry.RepositoryRegistry;

import java.util.Map;

/**
 * If mime-type is not includes, attempts to determine the mime-type from the URL and adds it into the provisioning
 * descriptor.
 */
public class MimeTypeFilter extends BundleFilter {

    public void validateBundle(ClientBundle clientBundle, Map content) throws StockingException {
        DescriptorFile descriptorFile = (DescriptorFile) clientBundle.getDescriptorFile();
        if (descriptorFile != null) {
            String mimeType = descriptorFile.getMimeType();
            if (mimeType == null) {
                RepositoryRegistry registry = (RepositoryRegistry)
                        filterTask.getStockingHandlerConfig().getStockingContext()
                                .getServletContext().getAttribute("org.jvending.registry.RepositoryRegistry");
                MimeTypeRepository mimeRepository = (MimeTypeRepository) registry.find("mimetype");
                mimeType = mimeRepository.getMimeTypeFromUri(descriptorFile.getValue());
            }
            if(mimeType != null) {
                descriptorFile.setMimeType(mimeType);
            }
        }
    }
}
