package org.jvending.provisioning.stocking.handler;

import org.jvending.provisioning.stocking.par.DescriptorFileType;
import org.jvending.provisioning.stocking.StockingException;
import org.jvending.registry.RepositoryRegistry;
import org.easymock.MockControl;
import junit.framework.TestCase;

import java.io.*;
import java.util.Map;
import java.util.HashMap;
import java.util.List;

public class JnlpHandlerTest extends TestCase {

    private JnlpHandler handler;

    private static String baseDir = System.getProperty("basedir");

    public void setUp() {
        RepositoryRegistry repositoryRegistry = RepositoryRegistry.Factory.create();
        try {
            repositoryRegistry.loadFromFile(baseDir + "/src/test/resources/registry-config.xml");
        } catch (IOException e) {
            e.printStackTrace();
            fail("Unable to load the RepositoryRegistry");
        }
        handler = new JnlpHandler();
        handler.setRepositoryRegistry(repositoryRegistry);
    }

    public void testDescriptorFile() {
        byte[] descriptor = null;
        try {
            FileInputStream fis = new FileInputStream(baseDir + "/src/test/resources/JnlpFile.jnlp");
            descriptor = copyStreamToBytes(fis);
        } catch (IOException e) {
            fail("Could not load test file");
        }

        Map content = getMockContentMap();
        content.put("/test.jnlp", descriptor);
        DescriptorFileType descriptorFile = getMockDescriptorFileType("/test.jnlp", "application/x-java-jnlp-file");
        try {
            List contentFiles = handler.getContentFiles(descriptorFile, content);
            assertEquals("Incorrect content size", 2, contentFiles.size());
        } catch (StockingException e) {
            fail("StockingException");
        }
    }

    private Map getMockContentMap() {
        Map contentMap = new HashMap();
        byte[] fakeBytes = {10, 12, 13};
        contentMap.put("/icon.jpg", fakeBytes);
        contentMap.put("/test.jar", fakeBytes);
        return contentMap;
    }

    private DescriptorFileType getMockDescriptorFileType(String url, String mimeType) {
        MockControl controlDescriptorFileType = MockControl.createControl(DescriptorFileType.class);
        DescriptorFileType descriptorFileType = (DescriptorFileType) controlDescriptorFileType.getMock();

        descriptorFileType.getValue();
        controlDescriptorFileType.setReturnValue(url);
        descriptorFileType.getValue();
        controlDescriptorFileType.setReturnValue(url);

        descriptorFileType.getMimeType();
        controlDescriptorFileType.setReturnValue(mimeType);

        controlDescriptorFileType.replay();
        return descriptorFileType;
    }

    private byte[] copyStreamToBytes(InputStream is) throws IOException {
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        byte[] buffer = new byte[1024];
        int n = 0;
        while ((n = is.read(buffer)) >= 0) {
            os.write(buffer, 0, n);
        }
        return os.toByteArray();
    }
}
